/* vRouter: network.cpp
 *
 *  Purpose: Encapsulate network and datalink objects that
 *           constitute a network
 *
 *   Changes:
 *  
 *   0.02 - Initial Release
 *
 */

#include "network.h"
#include "router.h"
#include "ipaddress.h"

#include <assert.h>
#include <vector>
#include <string>

/*
 * Constructor
 */
Network::Network()
{
  // Not much to do.
}

/* 
 * Destructor: Delete router and datalink objects on the network
 */
Network::~Network()
{

  cout << " ** DELETING NETWORK ** " << endl;
  for(unsigned int i = 0; i < routers.size(); i ++)
    delete routers[i];
  for(unsigned int i = 0; i < dataLinks.size(); i ++)
    delete dataLinks[i];

}

/* 
 *  Add a router to the network.  Never attempt
 *  to add a NULL router. 
 */
void Network::addRouter(Router *rtr)
{
  assert(rtr);
  
  routers.push_back(rtr);
}

/* 
 * Add a DataLink to the network.  Never call with
 * a NULL dataLink
 */
void Network::addDataLink(DataLink * dl)
{
  assert(dl);
  dataLinks.push_back(dl);
}

/* 
 * Return a pointer to a dataLink on the network, 
 * given the IPAddress of its network
 */
DataLink* Network::getDataLinkByAddress(IPAddress a)
{
  for(unsigned int i = 0; i < dataLinks.size(); i++)
    {
      if(dataLinks[i]->net() == a)
	return dataLinks[i];
    }
  return NULL;
}

/*
 * Return a pointer to a router on the network,
 * given its name
 */
Router* Network::getRouterByName(string name)
{
  for(unsigned int i = 0; i < routers.size(); i++)
    {
      if(routers[i]->name() == name)
	return routers[i];
    }
  return NULL;
}

/*
 * Remove all routers and datalinks from the network,
 * deleting datalink and router objects.
 */
void Network::clear()
{
  for(unsigned int i = 0; i < routers.size(); i ++)
    delete routers[i];
  for(unsigned int i = 0; i < dataLinks.size(); i ++)
    delete dataLinks[i]; 
  
  routers.clear();
  dataLinks.clear();
}
