/*
 * vRouter: Simulator
 *
 * Purpose: encapsulate simulator behavior into a class 
 *
 */

#include<vector>
#include<string>

#include "network.h"
#include "router.h"
#include "simulator.h"
#include "flow.h"
#include "ipaddress.h"

/*
 * Constructor
 */
Simulator::Simulator(Network *n)
{
  net = n;
  flows.clear();
}

/*
 * Add a flow to the simulation
 */
void Simulator::addFlow(Flow f)
{
  flows.push_back(f);
}

/*
 * Run the simulation
 */
void Simulator::runSim(int sliceWidth, int sliceCount)
{
  cout << " Running simulation: " << sliceCount << " slices of ";
  cout << sliceWidth << "ms each." << endl;
  
  for(int i =0; i < (sliceCount * sliceWidth); i+=sliceWidth)
    {
      cout << " ----------  " << i << " ms --------- " << endl;
      for(unsigned int q = 0; q < flows.size(); q++)
	{
	  if(flows[q].starttime() <= i && flows[q].duration() > 0)
	    {
	      flows[q].adjDuration(-sliceWidth);
	      
	      int bytesToSend =  int( flows[q].pktrate()  * flows[q].pktrate());
	      bytesToSend = int( (float) bytesToSend * ( (float) sliceWidth *.001));
	      bytesToSend += flows[q].freebytes;

	      int pktCount = bytesToSend / flows[q].pktsize();

	      flows[q].freebytes = bytesToSend - (pktCount * flows[q].pktsize());

	      
	      for(int l = 0; l < pktCount; l++)
		{
		  Packet *p = new Packet(flows[q].src(), flows[q].dst(), flows[q].srcRouter()->name() , flows[q].pktsize());
		  flows[q].packets.push_back(p);
		  flows[q].srcRouter()->deliverPacket(p);
		}
	    }
	}
      for(unsigned int j = 0; j < net->routers.size(); j++)
	{
	  net->routers[j]->rippleQueue();
	}
      for(unsigned int j = 0; j < net->routers.size(); j++)
	{
	  net->routers[j]->runTimeSlice(sliceWidth);
	}    

      for(unsigned int j =0; j < net->dataLinks.size(); j++)
	{
	  net->dataLinks[j]->runTimeSlice(sliceWidth);
	}
    }

  /* Now compute & print statistics */

  cout << "  ** Flow Analysis ** " << endl;
  for(unsigned int i = 0; i < flows.size(); i ++)
    {
      cout << "Flow from " << flows[i].srcRouter()->name();
      cout << " to " << flows[i].dst().toString() << ": " << endl;

       int pktCount = flows[i].packets.size();
       int deliveredCount = 0;
       int netDelay = 0;
       int netuDelay =0;
       for(int j = 0; j < pktCount; j++)
	 {
	   if(flows[i].packets[j]->delivered())
	     {
	       netDelay += flows[i].packets[j]->delay();
	       netuDelay += flows[i].packets[j]->udelay();
	       deliveredCount++;
	     }
	 }

       if(pktCount != 0)
	 {
	   cout << "   Packets delivered: ";
	   cout << (deliveredCount * 100 / pktCount) << "%" << endl;
	   float avgdelay = ((float) netDelay + ((float) netuDelay / 1000));
	   avgdelay /= pktCount;
	   cout << "   Average Delay    : " << avgdelay << "ms" << endl;
	 }
       else
	 {
	   cout << "   Error: No packets were delivered." << endl;
	 }
    }
}


